package jingdong

import (
	"context"
	"github.com/gogf/gf/container/garray"
	"github.com/gogf/gf/encoding/gjson"
	"github.com/gogf/gf/util/gconv"
)

type goodsJD struct {
}

var Goods = goodsJD{}

type CheckSaleRes struct {
	*CommonRes
	Result []struct {
		SkuId       int    `json:"skuid"`       //商品编号
		Name        string `json:"name"`        //销售名称
		SaleState   int    `json:"saleState"`   //是否可售 1：是，0：否
		IsCanVAT    int    `json:"isCanVAT"`    //是否可开增票，1：支持，0：不支持
		Is7ToReturn int    `json:"is7ToReturn"` //是否支持7天退货，1：是，0：不支持
	} `json:"result"`
}

type CheckOnSaleRes struct {
	*CommonRes
	Result []struct {
		Sku   int `json:"sku"`   //商品编号
		State int `json:"state"` //1：上架，0：下架
	} `json:"result"`
}

type GetStockRes struct {
	*CommonRes
	Result []struct {
		SkuID          int    `json:"skuId"`          //商品编号
		AreaID         string `json:"areaId"`         //入参时传入的区域码area。因京东目前是3、4级地址均支持，存在areaId在传入的3级地址后填充4级地址“_0“后返回的情况。
		StockStateID   int    `json:"stockStateId"`   //存状态编号。 参考枚举值： 33,39,40,36,34,99
		StockStateDesc string `json:"StockStateDesc"` //库存状态描述。以下为stockStateId不同时，此字段不同的返回值：
	} `json:"result"`
}

type CheckAreaLimitRes struct {
	*CommonRes
	Result []struct {
		SkuID          int  `json:"skuId"`          //商品编码
		IsAreaRestrict bool `json:"isAreaRestrict"` //true 代表区域受限 false 区域不受限
	} `json:"result"`
}

type GetUnionBalanceRes struct {
	*CommonRes
	Result struct {
		Balance struct {
			Pin         string  `json:"pin"`
			RemainLimit float64 `json:"remainLimit"`
		} `json:"balance"`
	}
}

type GetProductsPriceRes struct {
	*CommonRes
	Result []struct {
		SkuID       int     `json:"skuId"`
		Price       float64 `json:"price"`
		JdPrice     float64 `json:"jdPrice"`
		MarketPrice float64 `json:"marketPrice"`
	} `json:"result"`
}

// CheckSale 商品可售验证接口
func (goodsJD) CheckSale(ctx context.Context, req *garray.Array) (res *CheckSaleRes, err error) {
	method := "product/check"
	param := map[string]string{
		"skuIds": req.Join(","),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}

	err = gjson.New(result).Scan(&res)
	return
}

// CheckOnSale 查询商品上下架状态
func (goodsJD) CheckOnSale(ctx context.Context, req *garray.Array) (res *CheckOnSaleRes, err error) {
	method := "product/skuState"
	param := map[string]string{
		"sku": req.Join(","),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

//GetStock  查询商品库存
func (goodsJD) GetStock(ctx context.Context, req *garray.Array, area string) (res *GetStockRes, err error) {
	method := "stock/getNewStockById"
	param := map[string]string{
		"skuNums": req.Join(","),
		"area":    area,
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

// CheckAreaLimit 查询商品区域购买限制
func (goodsJD) CheckAreaLimit(ctx context.Context, skuIds *garray.Array, provinceID, cityID, countyID, townID interface{}) (res *CheckAreaLimitRes, err error) {
	method := "product/checkAreaLimit"
	param := map[string]string{
		"skuIds":   skuIds.Join(","),
		"province": gconv.String(provinceID),
		"city":     gconv.String(cityID),
		"county":   gconv.String(countyID),
		"town":     gconv.String(townID),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}

	err = gjson.New(result).Scan(&res)
	return
}

//GetProductsPrice 商品价格
func (goodsJD) GetProductsPrice(ctx context.Context, skus *garray.Array) (res *GetProductsPriceRes, err error) {
	method := "price/getSellPrice"
	param := map[string]string{
		"sku":       skus.Join(","),
		"queryExts": "marketPrice",
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}
