package jingdong

import (
	"context"
	"github.com/gogf/gf/encoding/gjson"
	"github.com/gogf/gf/frame/g"
	"github.com/gogf/gf/util/gconv"
)

type refundJD struct {
}

var Refund = refundJD{}

type RefundCancelRes struct {
	*CommonRes
	Result bool `json:"result"`
}

type ApplyReq struct {
	JdOrderId             uint                    `json:"jdOrderId"`
	CustomerExpect        uint                    `json:"customerExpect"`
	QuestionDesc          string                  `json:"questionDesc"`
	IsNeedDetectionReport bool                    `json:"isNeedDetectionReport"`
	IsHasPackage          bool                    `json:"isHasPackage"`
	QuestionPic           string                  `json:"questionPic"`
	PackageDesc           int                     `json:"packageDesc"`
	UserInfoDto           *AfterSaleCustomerDto   `json:"asCustomerDto"`
	PickDto               *AfterSalePickwareDto   `json:"asPickwareDto"`
	ReturnDto             *AfterSaleReturnwareDto `json:"asReturnwareDto"`
	DetailDto             *AfterSaleDetailDto     `json:"asDetailDto"`
}

type AfterSaleCustomerDto struct {
	Name  string `json:"customerContactName"` //联系人，最多50字符，
	Tel   string `json:"customerTel"`         //联系电话，最多50字符
	Phone string `json:"customerMobilePhone"` //手机号，最多50字符
}

type AfterSalePickwareDto struct {
	Type     int    `json:"pickwareType"`     //取件方式：4上门取件； 7 客户送货；40客户发货
	Province int    `json:"pickwareProvince"` //取件省，必填
	City     int    `json:"pickwareCity"`     //取件市，必填
	County   int    `json:"pickwareCounty"`   //取件县，必填
	Village  int    `json:"pickwareVillage"`  //取件乡镇
	Address  string `json:"pickwareAddress"`  //取件街道地址，取件方式为4（上门取件）时必填，最多500字符
}

type AfterSaleReturnwareDto struct {
	Type     int    `json:"returnwareType"`     //返件方式：自营配送(10),第三方配送(20);
	Province int    `json:"returnwareProvince"` //返件省
	City     int    `json:"returnwareCity"`     //返件市
	County   int    `json:"returnwareCounty"`   //返件县
	Village  int    `json:"returnwareVillage"`  //返件乡镇
	Address  string `json:"returnwareAddress"`  //返件街道地址，最多500字符
}

type AfterSaleDetailDto struct {
	Id  int  `json:"skuId"`  //商品编号
	Num uint `json:"skuNum"` //商品申请数量
}

type GetServiceListReq struct {
	JdOrderID string `json:"jdOrderId"`
	PageIndex int    `json:"pageIndex"`
	PageSize  int    `json:"pageSize"`
}

type GetServiceListRes struct {
	*CommonRes
	Result struct {
		TotalNum        int `json:"totalNum"`
		PageSize        int `json:"pageSize"`
		PageNum         int `json:"pageNum"`
		PageIndex       int `json:"pageIndex"`
		ServiceInfoList []struct {
			AfsServiceId int `json:"afsServiceId"`
			WareId       int `json:"wareId"`
			Cancel       int `json:"cancel"`
		} `json:"serviceInfoList"`
	} `json:"result"`
}

type GetServiceDetailRes struct {
	*CommonRes
	Result struct {
		AfsServiceId                    int    `json:"afsServiceId"`
		CustomerExpect                  int    `json:"customerExpect"`
		AfsApplyTime                    string `json:"afsApplyTime"`
		OrderId                         int    `json:"orderId"`
		ApproveNotes                    string `json:"approveNotes"`
		ApprovedResult                  int    `json:"approvedResult"`
		AfsServiceStep                  int    `json:"afsServiceStep"`
		ProcessResult                   int    `json:"processResult"`
		ServiceAftersalesAddressInfoDTO struct {
			Address  string `json:"address"`
			Tel      string `json:"tel"`
			LinkMan  string `json:"linkMan"`
			PostCode string `json:"postCode"`
		} `json:"serviceAftersalesAddressInfoDto"`
		ServiceFinanceDetailInfoDTOs []struct {
			ServiceFinanceDetailInfoDTO
			RefundPrice float64 `json:"refundPrice"`
		} `json:"serviceFinanceDetailInfoDTOs"`
	} `json:"result"`
}

type ServiceFinanceDetailInfoDTO struct {
	RefundWay     int    `json:"refundWay"`
	RefundWayName string `json:"refundWayName"`
	Status        int    `json:"status"`
	StatusName    string `json:"statusName"`
	WareName      string `json:"wareName"`
	WareId        int    `json:"wareId"`
}

type RefundCommonReq struct {
	OrderID string `json:"jdOrderId"`
	SkuID   string `json:"skuId"`
}

type CheckIsSupportRes struct {
	*CommonRes
	Result int `json:"result"`
}

type GetSupportTypeRes struct {
	*CommonRes
	Result []*CodeItem `json:"result"`
}

type CodeItem struct {
	Code string `json:"code"`
	Name string `json:"name"`
}

type GetReturnTypeRes struct {
	*CommonRes
	Result []*CodeItem `json:"result"`
}

// UpdateSendSku 根据填写单号
func (refundJD) UpdateSendSku(ctx context.Context, postData string) (res *CommonRes, err error) {
	method := "afterSale/updateSendSku"
	param := g.MapStrStr{
		"param": postData,
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

//AuditCancel 取消服务单
func (refundJD) AuditCancel(ctx context.Context, postData string) (res *CommonRes, err error) {
	method := "afterSale/auditCancel"
	param := g.MapStrStr{
		"param": postData,
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

//RefundCancel 取消售后
func (refundJD) RefundCancel(ctx context.Context, id interface{}) (res *RefundCancelRes, err error) {
	method := "afterSale/auditCancel"
	req := g.Map{
		"serviceIdList": gconv.Ints(id),
		"approveNotes":  "取消售后",
	}
	param := g.MapStrStr{
		"param": gjson.New(req).MustToJsonString(),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

// Apply 申请售后服务（退货、换货、维修）
func (refundJD) Apply(ctx context.Context, req *ApplyReq) (res *CommonRes, err error) {
	method := "afterSale/createAfsApply"
	param := g.MapStrStr{
		"param": gjson.New(req).MustToJsonString(),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

// GetServiceList 查询服务单根据客户账号和订单号分页查询服务单概要信息
func (refundJD) GetServiceList(ctx context.Context, req string) (res *GetServiceListRes, err error) {
	method := "afterSale/getServiceListPage"
	param := g.MapStrStr{
		"param": gjson.New(&GetServiceListReq{
			JdOrderID: req,
			PageIndex: 1,
			PageSize:  100,
		}).MustToJsonString(),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

// GetServiceDetail 根据服务单号查询服务单明细信息
func (refundJD) GetServiceDetail(ctx context.Context, req string) (res *GetServiceDetailRes, err error) {
	method := "afterSale/getServiceDetailInfo"
	param := g.MapStrStr{
		"param": req,
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

// CheckIsSupport 查询订单商品是否允许售后
func (refundJD) CheckIsSupport(ctx context.Context, req *RefundCommonReq) (res *CheckIsSupportRes, err error) {
	method := "afterSale/getAvailableNumberComp"
	param := g.MapStrStr{
		"param": gjson.New(req).MustToJsonString(),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

// GetSupportType 查询订单商品售后类型
func (refundJD) GetSupportType(ctx context.Context, req *RefundCommonReq) (res *GetSupportTypeRes, err error) {
	method := "afterSale/getCustomerExpectComp"
	param := g.MapStrStr{
		"param": gjson.New(req).MustToJsonString(),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}

	_ = gjson.New(result).Scan(&res)
	return
}

// GetReturnType 查询支持的商品返回京东方式
func (refundJD) GetReturnType(ctx context.Context, req *RefundCommonReq) (res *GetReturnTypeRes, err error) {
	method := "afterSale/getWareReturnJdComp"
	param := g.MapStrStr{
		"param": gjson.New(req).MustToJsonString(),
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}
